/*
 * Test dataspace operations.
 *
 * Copyright (C) 2020, 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <ipc/server.h>

#include "dataspace_server.h"
#include "simple_pager.h"



/* Component interface. */

class DataspaceServer : public SimplePager
{
public:
  explicit DataspaceServer(Memory *memory)
  : SimplePager(memory)
  {
    memset((void *) _region->start, (int) 'a', PAGE_SIZE);
  }
};



int main(void)
{
  /* Some memory. */

  Memory memory(10);

  /* Dataspace encapsulation. */

  DataspaceServer obj(&memory);

  /* Server capability. */

  l4_cap_idx_t server;

  /* Register a server associating it with the given object. */

  long err = ipc_server_bind("server", (l4_umword_t) &obj, &server);

  if (err)
  {
    printf("Could not bind server: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Wait for messages, dispatching to the handler. */

  ipc_server_loop(Dataspace_expected_items, &obj,
                  (ipc_server_handler_type) handle_Dataspace);

  return 0;
}
